@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::Recurse;
 
 #
-# $Id: Recurse.pm 1468 2016-03-21 13:11:35Z willem $
+# $Id: Recurse.pm 1470 2016-03-24 18:42:15Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1468 $)[1];
+$VERSION = (qw$LastChangedRevision: 1470 $)[1];
 
 
 =head1 NAME
@@ -143,13 +143,12 @@ sub send {
 		unless ( ref $ns ) {
 			$self->_diag("find missing glue for $ns");
 			$ns = [$res->nameservers($ns)];		# substitute IP list in situ
+		} else {
+			$res->nameservers(@$ns);		# cached IP list
 		}
 
-		$res->nameservers( map @$_, grep ref($_), @$nslist );
-
 		my $reply = $res->send($query);
 		next unless $reply;				# uncoverable branch true
-		next unless $reply->header->rcode eq 'NOERROR'; # uncoverable branch true
 
 		$self->_callback($reply);
 		return $reply;
